;
;    
;    LNK Library For Blitz
;    Analyses the properties of windows shortcut (.lnk) files in depth,
;    And provides an interface To create New shortcuts.    
;    Port of Version 3.01, 20090130
;
;    ---------------------------------------
;
;	This is a port of the lib I wrote for Cobra
;	See www.cobrabytes.com
;	
;	Because it was a quick copy, paste + hand-hack job, this Blitz source
;	is rather messy...
;
;	Pop over to www.SoCoder.net if you need any help with any code.
;
;    ---------------------------------------
;
;	Documentation: http://www.tophatstuff.co.uk/man/Blitz/Lnk/readme.html
;
;    ---------------------------------------
;       
;    Copyright (c) 2009 Ben Golightly
;
;    Permission is hereby granted, free of charge, To any person
;    obtaining a copy of this software And associated documentation
;    files (the "Software"), To deal in the Software without
;    restriction, including without limitation the rights To use,
;    copy, modify, merge, publish, distribute, sublicense, And/Or sell
;    copies of the Software, And To permit persons To whom the
;    Software is furnished To do so, subject To the following
;    conditions:
;
;    The above copyright notice And this permission notice shall be
;    included in all copies Or substantial portions of the Software.
;
;    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
;    EXPRESS Or IMPLIED, INCLUDING BUT Not LIMITED To THE WARRANTIES
;    OF MERCHANTABILITY, FITNESS For A PARTICULAR PURPOSE And
;    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS Or COPYRIGHT
;    HOLDERS BE LIABLE For ANY CLAIM, DAMAGES Or OTHER LIABILITY,
;    WHETHER IN AN ACTION OF CONTRACT, TORT Or OTHERWISE, ARISING
;    FROM, OUT OF Or IN CONNECTION WITH THE SOFTWARE Or THE USE Or
;    OTHER DEALINGS IN THE SOFTWARE.
;    
;    ---------------------------------------
;    
;    Credits:
;    
;    I've spent a lot of time examining different files in Hex editors
;    To create this unit, but I must thank the following source For the
;    invaluable help it provided:
;        
;        The Windows Shortcut File Format as reverse-engineered by Jesse Hager
;        Document Version 1.0
;        http://mediasrv.ns.ac.yu/extra/fileformat/windows/lnk/shortcut.pdf
;    
;    However, as a note To other people who may follow that guide, there were
;    some very important omissions that I have noted in my code as very important.
;    
;    Also, XVI is a great Hex editor For windows:
;    
;        Freeware Hex Editor XVI32
;        http://www.chmaas.handshake.de/delphi/freeware/xvi32/xvi32.htm
;    
;    ---------------------------------------
;    
;    Exported functions:
        
;        Function ReadShortcut(lnkpath: String)
;        Function CreateShortcut(lnkpath: String)
;        Function WriteShortcut() : Boolean
;
;	Exported type:
;
;		lnk
;

        
    ; Flag Properties
Const    LNK_SHELL_PRESENT               = 0 ; Export    // The shell item id list is present
Const    LNK_POINTS_TO_FILE_OR_DIR       = 1 ; Export    // Points to a file or directory
Const    LNK_HAS_DESCRIPTION             = 2 ; Export    // Has a description string
Const    LNK_HAS_RELATIVE_PATH           = 3 ; Export    // Has a relative path string
Const    LNK_HAS_WORKING_DIR             = 4 ; Export    // Has a working directory
Const    LNK_HAS_ARGUMENTS               = 5 ; Export    // Has command line arguments.
Const    LNK_HAS_CUSTOM_ICON             = 6 ; Export    // Has a custom icon / default icon
        
    ; Attribute Properties
Const    LNK_READONLY                    = 0 ; Export    // Target is read-only
Const    LNK_HIDDEN                      = 1 ; Export    // Target is hidden
Const    LNK_SYSTEM                      = 2 ; Export    // Target is a system file
Const    LNK_VOLUME_LABEL                = 3 ; Export    // Target is a volume label (impossible?)
Const    LNK_DIRECTORY                   = 4 ; Export    // Target is a directory
Const    LNK_HAS_BEEN_MODIFIED           = 5 ; Export    // Target has been modified since last backup. (archive)
Const    LNK_ENCRYPTED                   = 6 ; Export    // Target is encrypted (NTFS EFS)
Const    LNK_NORMAL                      = 7 ; Export    // Target is "normal"?
Const    LNK_TEMP                        = 8 ; Export    // Target is temporary
Const    LNK_SPARSE                      = 9 ; Export    // Target is a sparse file
Const    LNK_REPARSE_DATA                = 10 ; Export   // Target has reparse point data
Const    LNK_COMPRESSED                  = 11 ; Export   // Target is compressed
Const    LNK_OFFLINE                     = 12 ; Export   // Target is offline
    
    ; Volume Flags
Const    LNK_VOLUME_LOCAL                = 0 ; Export    // Available on a local volume
Const    LNK_VOLUME_NETWORK              = 1 ; Export    // Available on a network share
    
    ; Volume Types
Const    LNK_VOLUME_TYPE_UNKNOWN         = 0 ; Export
Const    LNK_VOLUME_TYPE_NO_ROOT         = 1 ; Export
Const    LNK_VOLUME_TYPE_REMOVABLE       = 2 ; Export    // Floppy, zip, etc
Const    LNK_VOLUME_TYPE_FIXED           = 3 ; Export    // Hard disk
Const    LNK_VOLUME_TYPE_REMOTE          = 4 ; Export    // Network drive
Const    LNK_VOLUME_TYPE_OPTICAL         = 5 ; Export    // CD / DVD
Const    LNK_VOLUME_TYPE_RAM             = 6 ; Export    // RAM Drive

    ; ShowWindow Modes (1,2 & 3 are available in the shortcut properties menu, some may be invalid)
Const    LNK_SWND_HIDE                   = 0 ; Export
Const    LNK_SWND_NORMAL                 = 1 ; Export
Const    LNK_SWND_SHOWMINIMIZED          = 2 ; Export
Const    LNK_SWND_SHOWMAXIMIZED          = 3 ; Export
Const    LNK_SWND_SHOWNOACTIVATE         = 4 ; Export
Const    LNK_SWND_SHOW                   = 5 ; Export
Const    LNK_SWND_MINIMIZE               = 6 ; Export
Const    LNK_SWND_SHOWMINNOACTIVE        = 7 ; Export
Const    LNK_SWND_SHOWNA                 = 8 ; Export
Const    LNK_SWND_RESTORE                = 9 ; Export
Const    LNK_SWND_SHOWDEFAULT            = 10 ; Export 
    

Type lnks
Field  lPath$             = ""
Field  lTargetPath$       = ""
Field  lRelativePath$     = ""
Field  lStartIn$          = ""
Field  lDescription$      = ""
Field  lWorkingDir$       = ""
Field  lCustomIcon$      = ""
Field  lArguments$        = "" 
Field  lIconIndex        = 0
Field  lFlag[7]
Field  lAttribute[13]
Field  lVolume[2]
Field  lVolumeType        = 0
Field  lVolumeSerial      = 0
Field  lVolumeLabel$       = "UNKNOWN"
Field  lShareName$         = ""
Field  lShowWnd           = LNK_SWND_NORMAL
Field  lCreationTime      = 0
Field  lModificationTime  = 0
Field  lLastAccessTime    = 0
Field  lTargetFileSize    = 0
Field  lHotKey            = 0
End Type ; Export

Global lnk.lnks
Global lnkResultString$ = "" ; Blitz functions can't return strings?



Function ReadShortcut(lnkpath$) ; Export
;{
;
;    Examines a windows .lnk shortcut file, And stores the results.
;      
;}
;Var
Local i = 0
Local lnkrfile ; Element
Local lnkbyte = 0
Local lnkdata$ = ""

;    { Lengths }    
Local sectlen = 0
Local tblelen = 0
Local itemlen = 0

;    { Positions }
Local pos_start = 0
Local pos_filelocinfo_start = 0

;    { Offsets }
Local offset = 0
Local offset_lvoltable = 0
Local offset_lvolbasepath = 0 
Local offset_nvoltable = 0
Local offset_filenamepath = 0
    
;Begin
If lnk = Null Then lnk.lnks = New lnks

    If Not (Right$(lnkpath$, 4) = ".lnk") Then lnkpath$ = lnkpath$ + ".lnk" 
    
            
;           { Load To memfile To reduce slow disk reads } - Can't do in Blitz? 
            lnkrfile = ReadFile(lnkpath$)
			If lnkrfile = 0 Then Return False
            lnk\lPath$ = lnkpath$
            
            
;            { *** HEADER *** }
            
;            { The First byte in a .lnk is always 4C, Or ASCII For the letter L }
            lnkbyte = ReadByte(lnkrfile)
            If Chr$(lnkbyte) = "L" Then
            
;                { Followed by 3 Null bytes }                      
                For i = 1 To 3
                    ReadByte(lnkrfile)
                Next
            
;                { Followed by a 16 byte GUID (00021401-0000-0000-00C0-000000000046) }
                For i = 1 To 4
                    ReadInt(lnkrfile)
                Next
            
;                { Followed by a byte For flags, which indicates what optional
;                parts of the file are present...
;            
;                    bit 6 : LNK_SHELL_PRESENT
;                    bit 5 : LNK_POINTS_TO_FILE_OR_DIR
;                    bit 4 : LNK_HAS_DESCRIPTION
;                    bit 3 : LNK_HAS_RELATIVE_PATH
;                    bit 2 : LNK_HAS_WORKING_DIR
;                    bit 1 : LNK_HAS_ARGUMENTS
;                    bit 0 : LNK_HAS_CUSTOM_ICON
;
;                    bit     0   1  2  3  4 5 6 7                
;                    value   128 64 32 16 8 4 2 1
;                      
;                }
            
                lnkbyte = ReadByte(lnkrfile)

                If Has(lnkbyte,1) Then      lnk\lFlag[LNK_SHELL_PRESENT]            = True
                If Has(lnkbyte,2) Then      lnk\lFlag[LNK_POINTS_TO_FILE_OR_DIR]    = True
                If Has(lnkbyte,4) Then      lnk\lFlag[LNK_HAS_DESCRIPTION]          = True
                If Has(lnkbyte,8) Then      lnk\lFlag[LNK_HAS_RELATIVE_PATH]        = True
                If Has(lnkbyte,16) Then     lnk\lFlag[LNK_HAS_WORKING_DIR]          = True
                If Has(lnkbyte,32) Then     lnk\lFlag[LNK_HAS_ARGUMENTS]            = True
                If Has(lnkbyte,64) Then     lnk\lFlag[LNK_HAS_CUSTOM_ICON]          = True            
            
;                { Followed by 3 Null bytes, possibly placeholders For future flags }
                For i = 1 To 3
                    ReadByte(lnkrfile)
                Next
            
;                { Followed by two bytes For attributes.
 ;                   If the shortcut target is Not a file Or directory
;                    (LNK_POINTS_TO_FILE_OR_DIR = False), Then these bytes are empty.
;
;                    byte 1            
;                    bit 7 : LNK_READONLY
;                    bit 6 : LNK_HIDDEN
;                    bit 5 : LNK_SYSTEM
;                    bit 4 : LNK_VOLUME_LABEL
;                    bit 3 : LNK_DIRECTORY
;                    bit 2 : LNK_HAS_BEEN_MODIFIED
;                    bit 1 : LNK_ENCRYPTED
;                    bit 0 : LNK_NORMAL
;                
;                    byte 2
;                    bit 7 : LNK_TEMP
;                    bit 6 : LNK_SPARSE
;                    bit 5 : LNK_REPARSE_DATA
;                    bit 4 : LNK_COMPRESSED
;                    bit 3 : LNK_OFFLINE
;
;                    bit     0   1  2  3  4 5 6 7                
;                    value   128 64 32 16 8 4 2 1
;                      
;                }
                        
                If lnk\lFlag[LNK_POINTS_TO_FILE_OR_DIR] Then     
            
                    lnkbyte = ReadByte(lnkrfile)
                    If Has(lnkbyte,1) Then      lnk\lAttribute[LNK_READONLY]             = True
                    If Has(lnkbyte,2) Then      lnk\lAttribute[LNK_HIDDEN]               = True
                    If Has(lnkbyte,4) Then      lnk\lAttribute[LNK_SYSTEM]               = True
                    If Has(lnkbyte,8) Then      lnk\lAttribute[LNK_VOLUME_LABEL]         = True
                    If Has(lnkbyte,16) Then     lnk\lAttribute[LNK_DIRECTORY]            = True
                    If Has(lnkbyte,32) Then     lnk\lAttribute[LNK_HAS_BEEN_MODIFIED]    = True
                    If Has(lnkbyte,64) Then     lnk\lAttribute[LNK_ENCRYPTED]            = True                
                    If Has(lnkbyte,128) Then    lnk\lAttribute[LNK_NORMAL]               = True
                
                    lnkbyte = ReadByte(lnkrfile)
                    If Has(lnkbyte,1) Then      lnk\lAttribute[LNK_TEMP]                 = True
                    If Has(lnkbyte,2) Then      lnk\lAttribute[LNK_SPARSE]               = True
                    If Has(lnkbyte,4) Then      lnk\lAttribute[LNK_REPARSE_DATA]         = True
                    If Has(lnkbyte,8) Then      lnk\lAttribute[LNK_COMPRESSED]           = True
                    If Has(lnkbyte,16) Then     lnk\lAttribute[LNK_OFFLINE]              = True
            
                Else
;                    // Skip two bytes...
                    ReadByte(lnkrfile)
                    ReadByte(lnkrfile)
                EndIf

;                { Followed by 2 Null bytes, possibly placeholders For future attributes }
                For i = 1 To 2
                    ReadByte(lnkrfile)
                Next
            
;                { Followed by three 64 bit integers For the dates (skipping For now) }
                For i = 1 To (3*2)
                    ReadInt(lnkrfile)
                Next
           
;                { Followed by an integer which contains the length of the target file in bytes }
                lnk\lTargetFileSize = ReadInt(lnkrfile)
            
;                { Followed by an integer which contains the index of the icon To use (zero If no custom icon) }
                lnk\lIconIndex  = ReadInt(lnkrfile)
            
;                { Followed by an integer which contains the ShowWnd value To pass }
                lnk\lShowWnd  = ReadInt(lnkrfile)
            
;                { Followed by an integer which specifies the hotkey assigned To the shortcut
;                    I don't know how To convert this integer into the hotkey,
;                    but I assume there's a list somewhere If you're interested
;                    in this functionality.                
;                }
                lnk\lHotKey  = ReadInt(lnkrfile)
            
;                { Followed by eight Null bytes, probably reserved For the future }
                For i = 1 To 2
                    ReadInt(lnkrfile)
                Next
                
                
                
;                { *** BODY *** }
                
;                { Shell Item ID List }                
                If lnk\lFlag[LNK_SHELL_PRESENT] Then
                    
;                    { Short integer containing the Length of the list }
                    sectlen = ReadShort(lnkrfile)
                    
;                    { Stub... I don't yet know what information is contained
;                        in this list, but by examining the file with a Hex
;                        editor, you can see that this section does hold
;                        a fair ammount of information... I plan To go back
;                        To this part.
;                    }
                    SeekFile(lnkrfile, FilePos(lnkrfile) + sectlen)
                
                EndIf
                
                
;                { File Location Info : Always present, but length may be zero
;                    If it's Not present, Then it's a pretty crap shortcut,
;                    because it won't point To anything!!!
;                }
                
;                    { Integer containing Length of this structure
;                        When lnk.lFlag[LNK_POINTS_TO_FILE_OR_DIR] = False, this
;                        should also be zero Length. I don't know If I should
;                        force this behaviour, Or rely on shortcuts To be valid.
;                    }
                    pos_filelocinfo_start = FilePos(lnkrfile)
                    sectlen = ReadInt(lnkrfile)
                    
                    If (sectlen > 0)
                    
;                        { Integer containing offset at which this structure ends
;                            Should be 1C (28) normally
;                        }
                        ReadInt(lnkrfile)
                                           
;                        { Followed by a byte For flags, which indicates what Type of
;                        volumes the file is available on.
;            
;                            bit 7 : LNK_VOLUME_LOCAL
;                            bit 6 : LNK_VOLUME_NETWORK
;
;                            bit     0   1  2  3  4 5 6 7                
;                            value   128 64 32 16 8 4 2 1 
;                      
;                        }
                        
                        lnkbyte = ReadByte(lnkrfile)
                        If Has(lnkbyte,1) Then    lnk\lVolume[LNK_VOLUME_LOCAL]   = True
                        If Has(lnkbyte,2) Then    lnk\lVolume[LNK_VOLUME_NETWORK] = True
            
;                        { Followed by 3 Null bytes, possibly placeholders For future flags }
                        For i = 1 To 3
                            ReadByte(lnkrfile)
                        Next
;                       
;                        { Followed by an integer containing the offset To the Local volume table
;                            This seems To always be 28, And must be relative To the beginning
;                            of the file location info structure, i.e. the offset at which
;                            this structure ends (the table seems To follow on from this
;                            structure) 
;                        }
                        offset_lvoltable = ReadInt(lnkrfile)
                        If lnk\lVolume[LNK_VOLUME_LOCAL] = False Then offset_lvoltable = 0
                        
;                        { Followed by an integer containing the offset To the base path on the Local volume }
                        offset_lvolbasepath = ReadInt(lnkrfile)
                        If lnk\lVolume[LNK_VOLUME_LOCAL] = False Then offset_lvolbasepath = 0
                        
;                        { Followed by an integer containing the offset To the network volume table }
                        offset_nvoltable = ReadInt(lnkrfile)
                        If lnk\lVolume[LNK_VOLUME_NETWORK] = False Then offset_nvoltable = 0
                        
;                        { Followed by an integer containing the offset To the remaining path of the filename }
                        offset_filenamepath = ReadInt(lnkrfile)
;                        
;                        { Filename on Local volume = basepath + final path
;                            Filename on network volume = sharename + final path
;                        }
;                        
                        
;                        { Local Volume Table }
                        If offset_lvoltable > 0 Then
;
 ;                           { Skip To offset
;                                The Local volume table usually appears After the File Location Info,
;                                but I don't know If you can always assume that it will. Therefore,
;                                It's easier To seek.
;                            }                        
                            SeekFile(lnkrfile, pos_filelocinfo_start + offset_lvoltable)
                        
;                            { Followed by an Integer containing the Length of this table,
;                                including the variable Length String at the End
;                            }
                            pos_start = FilePos(lnkrfile)
                            tblelen = ReadInt(lnkrfile)
                            
;                            { Then an integer containing the Type of the volume }
                            lnk\lVolumeType = ReadInt(lnkrfile)
                            
;                            { Then an integer containing the volume serial number }
                            lnk\lVolumeSerial = ReadInt(lnkrfile)
                            
;  ;                          { Followed by an integer containing an offset To the volume name
    ;                            This should always be 10 (Hex), e.g. 16, And is
 ;                               relative To the beginning of the Local volume
  ;                              table.
;                            }
                            offset = ReadInt(lnkrfile)
                            offset = (pos_start + offset) - FilePos(lnkrfile)
                            offset = offset 
                            
                            For i = 1 To (offset)
                                ReadByte(lnkrfile)
                            Next
                            
;                            { Then just Read off the volume label Until the End
;                                of the structure...
;                            }
                            lnk\lVolumeLabel$ = ""
                            offset = tblelen - (FilePos(lnkrfile) - pos_start)
                            offset = offset - 1 ;// Null terminated 
                            
                            For i = 1 To offset
                                lnk\lVolumeLabel$ = lnk\lVolumeLabel$ + Chr$(ReadByte(lnkrfile))
                            Next
                            
;                            { Null terminated byte... terminated! }
                            ReadByte(lnkrfile)
                            
                        EndIf
 ;                       { End Local Volume Table }  
                        
                        
  ;                      { Network Volume Table }
                        If offset_nvoltable > 0 Then
                        
;                            { Skip To offset }                            
                            SeekFile(lnkrfile, pos_filelocinfo_start + offset_nvoltable)    
                        
;                            { Followed by an Integer containing the Length of this table,
;                                including the variable Length String at the End
;                            }
                            pos_start = FilePos(lnkrfile)
                            tblelen = ReadInt(lnkrfile)
                            
 ;                           { Unknown Integer }
                            ReadInt(lnkrfile)
                            
  ;                          { Followed by an integer containing an offset To the share name
;   ;                             This should always be 14 (Hex), e.g. 20, And is
;                                relative To the beginning of the network volume
;                                table.
;                            }
                            offset = ReadInt(lnkrfile)
                            offset = (pos_start + offset) - FilePos(lnkrfile)
                            offset = offset 
                            
                            For i = 1 To (offset)
                                ReadByte(lnkrfile)
                            Next
                            
;                            { Then just Read off the volume label Until the End
;                                of the structure...
;                            }
                            lnk\lShareName$ = ""
                            offset = tblelen - (FilePos(lnkrfile) - pos_start)
                            offset = offset - 1; // Null terminated 
                            
                            For i = 1 To offset
                                lnk\lShareName$ = lnk\lShareName$ + Chr$(ReadByte(lnkrfile))
                            Next
                            
;                            { Null terminated byte... terminated! }
                            ReadByte(lnkrfile)
                            
                        EndIf                       
;                        { End Network Volume Table }
                        

;                        { Target Path : Local } 
                        If offset_lvoltable > 0 Then
                            
 ;                           { Base }
                            SeekFile(lnkrfile, pos_filelocinfo_start + offset_lvolbasepath)
							ReadStringUntilNull(lnkrfile)
                            lnk\lTargetPath$ = lnkResultString$
                            
  ;                          { Remaining - this usually appears To be emtpy, however
   ;                             searching For this brings us To the correct position
    ;                            To continue reading other Data.
     ;                       }
                            SeekFile(lnkrfile, pos_filelocinfo_start + offset_filenamepath )
							ReadStringUntilNull(lnkrfile)
                            lnk\lTargetPath$ = lnk\lTargetPath$ + lnkResultString$
                        
                        EndIf
                        
                        
;                        { Target Path : Network }
                        If offset_nvoltable > 0 Then
                        
                            SeekFile(lnkrfile, pos_filelocinfo_start + offset_filenamepath)
							ReadStringUntilNull(lnkrfile)
                            lnk\lTargetPath$ = lnk\lShareName$ + lnkResultString$
                            
                        EndIf
                        
 ;                   { End File Location Info }
                    
                
;                    { Annoyingly, that every String from here on seems To have
;                        NULLs every other char, And a Length number that doesn't
;                        count these NULLs.
;                        
;                        Also, the Length number is now stored in a byte, instead of
;                        an integer... make up your mind, microsoft!!! :-)
;                    }
                    If lnk\lFlag[LNK_HAS_DESCRIPTION] Then                        
						ReadStringStructure(lnkrfile, False)
                        lnk\lDescription$ = lnkResultString$
                    EndIf

                    If lnk\lFlag[LNK_HAS_RELATIVE_PATH] Then
						ReadStringStructure(lnkrfile, False)
                        lnk\lRelativePath$ = lnkResultString$
                    EndIf                         

                    If lnk\lFlag[LNK_HAS_WORKING_DIR] Then
						ReadStringStructure(lnkrfile, False)
                        lnk\lWorkingDir$ = lnkResultString$                            
                    EndIf                          

                    If lnk\lFlag[LNK_HAS_ARGUMENTS] Then
						ReadStringStructure(lnkrfile, False)
                        lnk\lArguments$ = lnkResultString$
                    EndIf

                    If lnk\lFlag[LNK_HAS_CUSTOM_ICON] Then
						ReadStringStructure(lnkrfile, False)
                        lnk\lCustomIcon$ = lnkResultString$
                    EndIf
                        
                EndIf

            EndIf
        
            CloseFile(lnkrfile)
End Function



Function CreateShortcut(lnkpath$, volume, volume_type)
;{
;    Creates a Handle For a New shortcut
;    In this Blitz version, this just clears the only existing handle
;}
Local i = 0
;Begin

If lnk = Null Then lnk.lnks = New lnks
        
        If Not (Right$(lnkpath$, 4) = ".lnk") Then lnkpath$ = lnkpath$ + ".lnk"

        lnk\lPath$ = lnkpath$
		lnk\lTargetPath$ 		= ""
		lnk\lRelativePath$     = ""
		lnk\lStartIn$          = ""
		lnk\lDescription$      = ""
		lnk\lWorkingDir$       = ""
		lnk\lCustomIcon$       = ""
		lnk\lArguments$        = "" 
		lnk\lIconIndex        = 0
		lnk\lVolumeType        = 0
		lnk\lVolumeSerial      = 0
		lnk\lVolumeLabel$       = "UNKNOWN"
		lnk\lShareName$         = ""
		lnk\lShowWnd           = LNK_SWND_NORMAL
		lnk\lCreationTime      = 0
		lnk\lModificationTime  = 0
		lnk\lLastAccessTime    = 0
		lnk\lTargetFileSize    = 0
		lnk\lHotKey            = 0

		For i = 0 To 6
			lnk\lFlag[i] = 0
		Next
		For i = 0 To 12
			lnk\lAttribute[i] = 0
		Next
		For i = 0 To 1
			lnk\lVolume[i] = 0
		Next

End Function



Function WriteShortcut() ; Export
;{
;    Writes a shortcut To a file
;}
;Var
Local    i =0

;    { Writing }
Local    lnkwfile ;: Element
Local    lnkbyte = 0
Local    lnkdata$ = ""
    
;    { Target Attributes }
Local    fattr = 0
Local    target ;: Element
Local    targetfilesize = 0
    
;    { Lenghts }
Local    len_filelocationinfo = 0
Local    len_localvolumetable = 0

;Begin

	If FileType(lnk\lTargetPath$) > 0 Then 
   
;        { Initial Setup : Flags }
        lnk\lFlag[LNK_SHELL_PRESENT] = False ; No clue about this, yet
        lnk\lFlag[LNK_POINTS_TO_FILE_OR_DIR] = True ; When would a link *Not*?
        lnk\lFlag[LNK_HAS_DESCRIPTION] = (Len(lnk\lDescription$) > 0)
        lnk\lFlag[LNK_HAS_RELATIVE_PATH] = (Len(lnk\lRelativePath$) > 0)
        lnk\lFlag[LNK_HAS_WORKING_DIR] = (Len(lnk\lWorkingDir$) > 0)
        lnk\lFlag[LNK_HAS_ARGUMENTS] = (Len(lnk\lArguments$) > 0)
        lnk\lFlag[LNK_HAS_CUSTOM_ICON] = (Len(lnk\lCustomIcon$) > 0)
        
;        { Initial Setup : Attributes }
;        fattr = FileAttr(lnk\lTargetPath)
;		Don't know if Blitz can get FileAttributes? Just make it up!
;        lnk\lAttribute[LNK_READONLY] = ((fattr And CFA_READONLY) = CFA_READONLY)
		lnk\lAttribute[LNK_READONLY] = False
;        lnk\lAttribute[LNK_HIDDEN] = ((fattr And CFA_HIDDEN) = CFA_HIDDEN)
		lnk\lAttribute[LNK_HIDDEN] = False
        lnk\lAttribute[LNK_SYSTEM] = False
        lnk\lAttribute[LNK_VOLUME_LABEL] = False
;        lnk\lAttribute[LNK_DIRECTORY] = ((fattr And CFA_DIRECTORY) = CFA_DIRECTORY)
		lnk\lAttribute[LNK_DIRECTORY] = (FileType(lnk\lTargetPath$) = 2)
        lnk\lAttribute[LNK_HAS_BEEN_MODIFIED] = True ; Usually appears True
        lnk\lAttribute[LNK_ENCRYPTED] = False
        lnk\lAttribute[LNK_NORMAL] = False
        lnk\lAttribute[LNK_TEMP] = False
        lnk\lAttribute[LNK_SPARSE] = False
        lnk\lAttribute[LNK_REPARSE_DATA] = False
        lnk\lAttribute[LNK_COMPRESSED] = False
        lnk\lAttribute[LNK_OFFLINE] = False
        
;        { File Size }
;        target = ReadFile(lnk.lTargetPath)
        targetfilesize = FileSize(lnk\lTargetPath$)
;        CloseFile(target)          
            
;        { Write To a memfile To reduce slow disk access } 
;        lnkwfile = CreateMemFile()

		lnkwfile = WriteFile(lnk\lPath$)
		If lnkwfile = 0 Then Return False

;        { First byte is "L" }
        WriteByte(lnkwfile, Asc("L"))
        
;        { Three Null bytes }
        For i = 1 To 3
            WriteByte(lnkwfile, 0)
        Next
        
;        { 16 Byte GUID }
        WriteByte(lnkwfile, $01)
        WriteByte(lnkwfile, $14)
        WriteByte(lnkwfile, $02)
        WriteByte(lnkwfile, $00)
        WriteByte(lnkwfile, $00)
        WriteByte(lnkwfile, $00)
        WriteByte(lnkwfile, $00)
        WriteByte(lnkwfile, $00)
        WriteByte(lnkwfile, $C0)
        WriteByte(lnkwfile, $00)
        WriteByte(lnkwfile, $00)
        WriteByte(lnkwfile, $00)
        WriteByte(lnkwfile, $00)
        WriteByte(lnkwfile, $00)
        WriteByte(lnkwfile, $00)
        WriteByte(lnkwfile, $46)
        
;        { Followed by a byte For flags, which indicates what optional
;        parts of the file are present...
            
;            bit 6 : LNK_SHELL_PRESENT
;            bit 5 : LNK_POINTS_TO_FILE_OR_DIR
;            bit 4 : LNK_HAS_DESCRIPTION
;            bit 3 : LNK_HAS_RELATIVE_PATH
;            bit 2 : LNK_HAS_WORKING_DIR
;            bit 1 : LNK_HAS_ARGUMENTS
;            bit 0 : LNK_HAS_CUSTOM_ICON
;
;            bit     0   1  2  3  4 5 6 7                
;            value   128 64 32 16 8 4 2 1
;                      
;        }
            
        lnkbyte = 0

        If lnk\lFlag[LNK_SHELL_PRESENT]         Then lnkbyte = lnkbyte + 1
        If lnk\lFlag[LNK_POINTS_TO_FILE_OR_DIR] Then lnkbyte = lnkbyte + 2
        If lnk\lFlag[LNK_HAS_DESCRIPTION]       Then lnkbyte = lnkbyte + 4
        If lnk\lFlag[LNK_HAS_RELATIVE_PATH]     Then lnkbyte = lnkbyte + 8
        If lnk\lFlag[LNK_HAS_WORKING_DIR]       Then lnkbyte = lnkbyte + 16
        If lnk\lFlag[LNK_HAS_ARGUMENTS]         Then lnkbyte = lnkbyte + 32
        If lnk\lFlag[LNK_HAS_CUSTOM_ICON]       Then lnkbyte = lnkbyte + 64
        lnkbyte = lnkbyte + 128 ; This always seems To be there - REALLY IMPORTANT!!! 
        WriteByte(lnkwfile, lnkbyte)
            
;        { Followed by 3 Null bytes, possibly placeholders For future flags }
        For i = 1 To 3
            WriteByte(lnkwfile, 0)
        Next
        
;        { Followed by two bytes For attributes.
;            If the shortcut target is Not a file Or directory
;            (LNK_POINTS_TO_FILE_OR_DIR = False), Then these bytes are empty.
;
;            byte 1            
;            bit 7 : LNK_READONLY
;            bit 6 : LNK_HIDDEN
;            bit 5 : LNK_SYSTEM
;            bit 4 : LNK_VOLUME_LABEL
;            bit 3 : LNK_DIRECTORY
;            bit 2 : LNK_HAS_BEEN_MODIFIED
;            bit 1 : LNK_ENCRYPTED
;            bit 0 : LNK_NORMAL
;                
;            byte 2
;            bit 7 : LNK_TEMP
;            bit 6 : LNK_SPARSE
;            bit 5 : LNK_REPARSE_DATA
;            bit 4 : LNK_COMPRESSED
;            bit 3 : LNK_OFFLINE
;
;            bit     0   1  2  3  4 5 6 7                
;            value   128 64 32 16 8 4 2 1
;                      
;        }
;                        
        If lnk\lFlag[LNK_POINTS_TO_FILE_OR_DIR] Then    
            
            lnkbyte = 0
            If lnk\lAttribute[LNK_READONLY]             Then lnkbyte = lnkbyte + 1
            If lnk\lAttribute[LNK_HIDDEN]               Then lnkbyte = lnkbyte + 2
            If lnk\lAttribute[LNK_SYSTEM]               Then lnkbyte = lnkbyte + 4
            If lnk\lAttribute[LNK_VOLUME_LABEL]         Then lnkbyte = lnkbyte + 8
            If lnk\lAttribute[LNK_DIRECTORY]            Then lnkbyte = lnkbyte + 16
            If lnk\lAttribute[LNK_HAS_BEEN_MODIFIED]    Then lnkbyte = lnkbyte + 32
            If lnk\lAttribute[LNK_ENCRYPTED]            Then lnkbyte = lnkbyte + 64
            If lnk\lAttribute[LNK_NORMAL]               Then lnkbyte = lnkbyte + 128
            WriteByte(lnkwfile, lnkbyte)
                
            lnkbyte = 0
            If lnk\lAttribute[LNK_TEMP]                 Then lnkbyte = lnkbyte + 1
            If lnk\lAttribute[LNK_SPARSE]               Then lnkbyte = lnkbyte + 2
            If lnk\lAttribute[LNK_REPARSE_DATA]         Then lnkbyte = lnkbyte + 4
            If lnk\lAttribute[LNK_COMPRESSED]           Then lnkbyte = lnkbyte + 8
            If lnk\lAttribute[LNK_OFFLINE]              Then lnkbyte = lnkbyte + 16
            lnkbyte = lnkbyte + 32 ; This always seems To be there - REALLY IMPORTANT!!!
            WriteByte(lnkwfile, lnkbyte)
            
        Else
            ; Skip two bytes...
            WriteByte(lnkwfile, 0)
            WriteByte(lnkwfile, 0)
        EndIf
        
;        { Followed by 2 Null bytes, possibly placeholders For future attributes }
        For i = 1 To 2
            WriteByte(lnkwfile, 0)
        Next
            
;        { Followed by three 64 bit integers For the dates (skipping For now) }
        For i = 1 To (3*2)
            WriteInt(lnkwfile, 0)
        Next
        
 ;       { Followed by an integer which contains the length of the target file in bytes }       
        WriteInt(lnkwfile, targetfilesize)
            
;        { Followed by an integer which contains the index of the icon To use (zero If no custom icon) }
        WriteInt(lnkwfile, lnk\lIconIndex)
            
;        { Followed by an integer which contains the ShowWnd value To pass }
        WriteInt(lnkwfile, lnk\lShowWnd)
            
;        { Followed by an integer which specifies the hotkey assigned To the shortcut }
        WriteInt(lnkwfile, lnk\lHotKey)
            
;        { Followed by eight Null bytes, probably reserved For the future }
        For i = 1 To 2
            WriteInt(lnkwfile, 0)
        Next
        
;        { Shell Item Id List
;            To Do...        
;        }
        
;        { File Location Info }
        
;            { The First integer is the total Length of this structure, And all
;                following Data (including Local volume table And the base path) up
;                To the description/relative path/etc String.
;            }
                        
            len_localvolumetable = 16 + Len(lnk\lVolumeLabel$) + 1
            len_filelocationinfo = 28 + len_localvolumetable + Len(lnk\lTargetPath$) + 1 + 1
            
            WriteInt(lnkwfile, len_filelocationinfo)
            
;            { Pointer To First offset (LVT), should be 1C (28) }
            WriteInt(lnkwfile, 28)
            
;            { Followed by a byte For flags, which indicates what Type of
;                volumes the file is available on.
;            
;                bit 7 : LNK_VOLUME_LOCAL
;                bit 6 : LNK_VOLUME_NETWORK
;
;                bit     0   1  2  3  4 5 6 7                
;                value   128 64 32 16 8 4 2 1 
;                      
;            }
                        
            WriteByte(lnkwfile, 1) ;// Local
            
;            { Followed by 3 Null bytes, possibly placeholders For future flags }
            For i = 1 To 3
                WriteByte(lnkwfile, 0)
            Next
            
;            { Integer offset To the Local volume table - same as pointer To the
;                First offset, i.e. 1C/28
;            }
            WriteInt(lnkwfile, 28)

;            { Offset To the base path on the Local volume }            
            WriteInt(lnkwfile, 28 + len_localvolumetable)
            
;            { Integer offset To the network volume table - 0 in this Case }
            WriteInt(lnkwfile, 0)
            
;            { Offset To the remaining path }
            WriteInt(lnkwfile, 28 + len_localvolumetable + Len(lnk\lTargetPath$) + 1)
                                
            
;            { Local Volume Table }
            
;                { Length of this structure }
                WriteInt(lnkwfile, len_localvolumetable)
                
;                { Volume Type }
                If lnk\lVolumeType = 0 Then lnk\lVolumeType = LNK_VOLUME_TYPE_FIXED
                WriteInt(lnkwfile, lnk\lVolumeType)
                
;                { Volume Serial Number
;                    Does Not have To be valid. I'll use zero.
;                }
                WriteInt(lnkwfile, 0)
                
;                { Offset To the volume label (always 10 Hex, i.e. 16) }
                WriteInt(lnkwfile, 16)
                
;                { ... Which is a bit useless, because the volume label appears
;                straight After this offset! }
                WriteStringAppendNull(lnkwfile, lnk\lVolumeLabel$)
            
;            { End Local Volume Table }
            
            
;            { Target Path }
            WriteStringAppendNull(lnkwfile, lnk\lTargetPath$)
            
;            { Remaing Path - empty }
            WriteStringAppendNull(lnkwfile, "")
        
;        { End File Location Info }
        
;        { Strings }
        If lnk\lFlag[LNK_HAS_DESCRIPTION]       Then WriteStringStructure(lnkwfile, lnk\lDescription$)
        If lnk\lFlag[LNK_HAS_RELATIVE_PATH]     Then WriteStringStructure(lnkwfile, lnk\lRelativePath$)
        If lnk\lFlag[LNK_HAS_WORKING_DIR]       Then WriteStringStructure(lnkwfile, lnk\lWorkingDir$)
        If lnk\lFlag[LNK_HAS_ARGUMENTS]         Then WriteStringStructure(lnkwfile, lnk\lArguments$)
        If lnk\lFlag[LNK_HAS_CUSTOM_ICON]       Then WriteStringStructure(lnkwfile, lnk\lCustomIcon$)        
        
 ;       { End }
        WriteInt(lnkwfile, 0)

;        SaveMemFile(lnkwfile, lnk.lPath)       
		CloseFile(lnkwfile)
        Return True
    
    EndIf
	Return False
End Function





Function Has(a, b)
    Return ((a And b) = b)
End Function


Function ReadStringStructure(mfile, cint=True)
;Var
Local inbyte = 0
Local itemlen = 0
Local i=0
;Begin
lnkResultString$ = ""

 ;   { Integer containing Length of the String }                        
    If cint Then
        itemlen = ReadInt(mfile)
    Else
        itemlen = ReadByte(mfile) * 2
    EndIf
    
    If itemlen > 0 Then
    
        For i = 1 To itemlen
        
            inbyte = ReadByte(mfile)
            
            If inbyte > 0 Then lnkResultString$ = lnkResultString$ + Chr$(inbyte)
                
        Next
        
;        { Null terminated byte }
        ReadByte(mfile)
        
    Else
        lnkResultString$ = ""
    EndIf    
End Function


Function ReadStringUntilNull(mfile)
;Var
 Local inbyte = 0
;Begin

lnkResultString$ = ""

    inbyte = ReadByte(mfile)
    
    While (inbyte > 0)
        lnkResultString$ = lnkResultString$ + Chr$(inbyte)
        inbyte = ReadByte(mfile)
    Wend
    
    
End Function


Function WriteStringAppendNull(mfile, s$)
;Var
 Local i  = 0
;Begin

    If Len(s$) > 0 Then
        
        For i = 1 To Len(s$)
            WriteByte(mfile, Asc(Mid(s$,i,1)))
        Next
    
    EndIf
    
    WriteByte(mfile, 0) ;// Null terminated
    
    
End Function

Function WriteStringStructure(mfile, s$)
;Var
Local l = 0
Local i = 0
;Begin

    l = Len(s)
    WriteByte(mfile, l)
    
    If Len(s$) > 0 Then
        
        For i = 1 To l
            WriteByte(mfile, 0)
            WriteByte(mfile, Asc(Mid(s$,i,1)))
        Next
    
    EndIf
    
    WriteByte(mfile, 0) ;// Null terminated   
    
End Function